<?php
/**
 * 
 * Questo file da chiamare direttamente costruisce il corretto link di pagamento
 * verso il gateway richiesto e per il file rchiesto.
 * L'id del file viene passato con il parametro "f", il gateway con il parametro 
 * "g" e scritto per esteso: "paypal", "plimus", "clickbank" (in minuscolo).
 *
 * Nel caso il chiamante abbia una sessione utente attiva, viene aggiunto anche il
 * campo custom che passa al gateway di pagamento l'id dell'utente per abbinargli la
 * vendita alla chiamata server to server di pagameto completato.
 *
 * TODO: estendere eventualmente il file in modo che abbia anch'esso una email business
 * specifica per PayPal con priorità su quella generale.
 * 
 * TODO: costruire tutta la gestione delle affiliazioni (in parte rea prevista).
 * 
 * TODO: sistemare lo switch sul sistema di pagamento che nel "default" cerca di creare il 
 * link quando non è specificato il gateway. Va fissato il controllo sul trattino come
 * prefisso e sul prezzo diretto in caso di paypal. Oppure eliminare tutto!
 */

include '../../../wp-load.php';

$file_commerce_pro->log_info('buy.php');
$file_commerce_pro->log_debug($_GET);


/* 
 * Costruisce l'URL per PayPal tenendo conto se il file è associato ad un bottone
 * oppure se associato ad un prezzo diretto.
 * Nel caso di prezzo diretto è necessario passare l'email del merchant, ma FCP ne
 * gestisce più di una per la verifica delle transazioni (vedi main settings): in
 * questo caso utilizza la prima.
 */
function file_commerce_pro_paypal_url($file) {
    global $file_commerce_pro;

    // Is it a button or a direct price?
    if (!empty($file->amount)) {

        $url = $file_commerce_pro->paypal_url . '?cmd=_xclick';

        $url .=
                '&item_number=' . $file->id .
                '&item_name=' . urlencode($file->title) .
                '&currency_code=' . $file->currency .
                '&amount=' . urlencode($file->amount);

        // Prima email merchant tra quelle inserite
        $emails = preg_split("/[\s,]+/", $file_commerce_pro->get_option('paypal_email'));
        $url .= '&business=' . $emails[0];
    } else {
        $url = $file_commerce_pro->paypal_url . '?cmd=_s-xclick';
        $url .= '&hosted_button_id=' . $file->paypal;
    }

    $user_id = get_current_user_id();
    if (!empty($user_id)) {
        $url .= '&custom=' . $user_id;
    }

    $url .= '&notify_url=' . urlencode(plugins_url('paypal.php', __FILE__)) .
            '&return=' . urlencode(plugins_url('thankyou.php', __FILE__));


    return $url;
}

/**
 * Costruisce l'URL per ClickBank.
 */
function file_commerce_pro_clickbank_url($file) {
    global $file_commerce_pro;
    $url = 'http://' . $file->clickbank . '.' . $file_commerce_pro->get_option($file_commerce_pro->option_name_clickbank_vendor_code) .
            '.pay.clickbank.net/';

    $user_id = get_current_user_id();
    if (!empty($user_id)) {
        $url .= '?user=' . $user_id;
    }

    return $url;
}

/**
 * Costruisce l'URL per Plimus.
 */
function file_commerce_pro_plimus_url($file) {
    global $file_commerce_pro;

    $url = 'https://www.plimus.com/jsp/buynow.jsp?contractId=' . trim($file->plimus);

    return $url;
}

// Recupera il file richiesto per la creazione del link di pagamento
$file = $file_commerce_pro->get_file($_REQUEST['f']);
$file_commerce_pro->log_info($file);
if (empty($file) || $file->status != FileCommercePro::FILE_ON_SALE) {
    echo 'File not found or no more on sale';
    die();
}

// Non gestita
$affiliateid = $_REQUEST['a'];

// Recupero del gateway e costruzione dell'URL di pagamento
$gateway = $_REQUEST['g'];

$url = '';
switch ($gateway) {

    case 'paypal':
        if ($file->paypal_enabled != 1) {
            echo 'PayPal is not accepted for this file';
            die();
        }
        $url = file_commerce_pro_paypal_url($file);
        break;

    case 'clickbank':
        if ($file->clickbank_enabled != 1) {
            echo 'Clickbank is not accepted for this file';
            die();
        }
        $url = file_commerce_pro_clickbank_url($file);
        break;

    case 'plimus':
        if ($file->plimus_enabled != 1) {
            echo 'Plimus is not accepted for this file';
            die();
        }
        $url = file_commerce_pro_plimus_url($file);
        break;
            
    default:
        if ($file->paypal_enabled == 1)
            $url = file_commerce_pro_paypal_url($file);
        else if ($file->clickbank_enabled == 1)
            $url = file_commerce_pro_clickbank_url($file);
        else if ($file->plimus_enabled == 1)
            $url = file_commerce_pro_plimus_url($file);
        else {
            echo 'There are no gateways available to buy the file';
            die();
        }
}

$file_commerce_pro->log_info('Url: ' . $url);

header('Location: ' . $url);
?>
