<?php
/**
 * Questo è il "listener" per le chiamate server to server di ClickBank e viene chiamato
 * direttamente e va inserito nella configurazione sulla console IPN di ClickBank.
 * 
 * TODO: aggiungere le notifica all'amministratore se qualcosa va male (vedi paypal.php).
 */

include '../../../wp-load.php';

/**
 * Create the cverify value and verify if it is correct using the plain text values in the HTTP POST
 * and your secret key (e.g. not URL-encoded). Return false on fail.
 */
function file_commerce_pro_clickbank_verify() {
    global $file_commerce_pro;

    $secretKey = $file_commerce_pro->get_option('clickbank_secret_key');
    $pop = "";
    $ipnFields = array();
    foreach ($_POST as $key => $value) {
        if ($key == "cverify")
            continue;
        $ipnFields[] = $key;
    }
    sort($ipnFields);
    foreach ($ipnFields as $field) {
        // On WordPress magic quotes are ALWAYS enabled
        $pop = $pop . stripslashes($_POST[$field]) . "|";
    }
    $pop = $pop . $secretKey;
    // On WordPress the internal encoding is forced to UTF-8, ClickBank data is ISO-8859-1
    $calcedVerify = sha1(mb_convert_encoding($pop, 'UTF-8', 'ISO-8859-1'));
    $calcedVerify = strtoupper(substr($calcedVerify, 0, 8));
    if ($calcedVerify == $_POST["cverify"]) return true;

    $file_commerce_pro->admin_mail('ClickBank IPN not verified!', "The IPN below was not verified, check the transaction on ClickBank administration side. With ClickBank you cannot resend a notification, you must create the contract (and maybe the user) manually. It's best to contact the customer suddenly.\n\n" . print_r($_POST, true));

    return false;
}

/**
 * Extract clickbank custom fields value and return as array
 * @param $string value of param cproditem
 */
function file_commerce_pro_clickbank_custom_fields($string) {
    $result = array();
    $values = explode('&', $string);
    foreach ($values AS $value) {
        $tmp = explode('=', $value);
        $result[$tmp[0]] = $tmp[1];
    }
    return $result;
}

$file_commerce_pro->log_info('clickbank.php');

$file_commerce_pro->log_debug($_GET);
$file_commerce_pro->log_debug($_POST);

if (!file_commerce_pro_clickbank_verify()) {
    $file_commerce_pro->log_error('ClickBank IPN unverified');
    die();
}

$txnid = $_POST['ctransreceipt'];
$cproditem = $_POST['cproditem'];

// Search the file by ClickBank item number
$file = $wpdb->get_row($wpdb->prepare("select * from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_FILES . " where clickbank=%s", $_POST['cproditem']));

if (empty($file)) {
    $file_commerce_pro->log_error('File not found!');
    die();
}

$file_commerce_pro->log_debug($file);

// Prepare the ipn data
$ipn = array();
$ipn['gateway'] = FileCommercePro::GATEWAY_CLICKBANK;
$ipn['data'] = serialize($_POST);
$ipn['request'] = serialize($_SERVER);
$ipn['created'] = time();

$customfieldsarray = file_commerce_pro_clickbank_custom_fields($_POST['cvendthru']);

switch ($_POST['ctransaction']) {
    case 'SALE':
    // The purchase of a standard product or the initial purchase of recurring billing product.
    case 'BILL':
        
        // Try to retreive the user
        $user = get_user_by('id', $customfieldsarray['user']);

        if (!$user) {
            $file_commerce_pro->log_debug('User not found, create a new user for email: ' . $_POST['ccustemail']);
            $user = $file_commerce_pro->create_user($_POST['ccustemail']);
        }

        $file_commerce_pro->log_debug($user);

        // A rebill for a recurring billing product.
        $contract = array();
        $contract['fileid'] = $file->id;
        $contract['userid'] = $user->ID;
        $contract['status'] = FileCommercePro::CONTRACT_VALID;
        $contract['gateway'] = FileCommercePro::GATEWAY_CLICKBANK;
        $contract['txnid'] = $txnid;
        $contract['amount'] = $_POST['corderamount'];
        $contract['net_amount'] = $_POST['caccountamount'];
        $contract['expire'] = $file_commerce_pro->get_contract_expiration_value($file->duration);
        $contract['name'] = stripslashes($_POST['ccustfullname']);
        $contract['email'] = $_POST['ccustemail'];
        $contract['token'] = md5(time() . rand());

        $contract = apply_filters('file_commerce_pro_contract', $contract);
        $r = $wpdb->insert($wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS, $contract);
        $contract['id'] = $wpdb->insert_id;
        
        if (!$r)
            $file_commerce_pro->log_error("Error saving contract to database");
        else
            $file_commerce_pro->log_info("Saved contract: user id '$userid' file id '" . $file->id . "'");

        $ipn['contractid'] = $contract['id'];
        $wpdb->insert($wpdb->prefix . FILECOMMERCEPRO_TABLE_IPN, $ipn);

        // Send the email anyway so the customer receive something even in case of error
        $file_commerce_pro->notify_new_contract($contract, $user);
        die();
        break;

    case 'RFND':
    case 'CGBK':
        $contract = $wpdb->get_row($wpdb->prepare("select * from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS . " where gateway='" . FileCommercePro::GATEWAY_CLICKBANK . "' and txnid=%s", $txnid), ARRAY_A);
        if (empty($contract)) {
            $file_commerce_pro->log_error('Contract not found.');
            $wpdb->insert($wpdb->prefix . FILECOMMERCEPRO_TABLE_IPN, $ipn);
            die();
        }

        $ipn['contractid'] = $contract['id'];
        $wpdb->insert($wpdb->prefix . FILECOMMERCEPRO_TABLE_IPN, $ipn);
        
        $wpdb->update($wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS, array('status' => FileCommercePro::CONTRACT_INVALID), array('id' => $contract['id']));
        $file_commerce_pro->notify_invalidated_contract($contract);
        die();
        break;

    case 'CGBK':
        //A chargeback for a standard or recurring product.

        break;
    case 'INSF':
        //An eCheck chargeback for a standard or recurring product.

        break;
    case 'CANCEL-REBILL':
        //The cancellation of a recurring billing product. Recurring billing products that are canceled do not result in any other action.

        break;
    case 'UNCANCEL-REBILL':
        //Reversing the cancellation of a recurring billing product.

        break;
    case 'TEST_SALE':
        // To test approvation...
//        $contract = array();
//        $contract['fileid'] = $file->id;
//        $contract['userid'] = $user->ID;
//        $contract['status'] = FileCommercePro::CONTRACT_VALID;
//        $contract['gateway'] = FileCommercePro::GATEWAY_CLICKBANK;
//        $contract['txnid'] = $trackingid;
//        $contract['amount'] = $_POST['corderamount'];
//        $contract['net_amount'] = $_POST['caccountamount'];
//        $contract['expire'] = $file_commerce_pro->get_contract_expiration_value($file->duration);
//        $wpdb->insert($wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS, $contract);
//        $contract['id'] = $wpdb->insert_id;
//        //Triggered by using the test link on the site page.
//        $file_commerce_pro->log_debug("clickbank notification: test transaction " . print_r($REQUEST, 1));
        break;
    default:
}

/*
http://www.clickbank.com/help/vendor-help/vendor-basics/creating-a-payment-link/
http://www.clickbank.com/help/account-help/account-tools/instant-notification-service/

http://www.ilsitodiluca.it/commerce/?plugin=file-commerce-pro&action=clickbankthankyoupage&item=1&cbreceipt=KGNHEPWB&time=1301000934&cbpop=1832FEE2&cbaffi=0&cname=pippo+pluto&cemail=luca.iovio%40gmail.com&ccountry=IT&czip=36100&user=1
*/


