<?php

define('FILECOMMERCEPRO_PATH', dirname(__FILE__) . '/');
define('FILECOMMERCEPRO_FILE_UPLOAD_DIR', "file-commerce-pro/");

define('FILECOMMERCEPRO_TABLE_FILES', 'filecommercepro_files');
define('FILECOMMERCEPRO_TABLE_CONTRACTS', 'filecommercepro_contracts');
define('FILECOMMERCEPRO_TABLE_IPN', 'filecommercepro_ipn');
define('FILECOMMERCEPRO_TABLE_DOWNLOADS', 'filecommercepro_downloads');

/**
 * Returns file data from table
 * @param $id
 */
function file_commerce_pro_load_file_from_db($id) {
    global $wpdb;
    $result = false;
    if (is_numeric($id))
    {
    	$query = "select * from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_FILES . " where id = " . $id;
		$files = $wpdb->get_results($query);
		$result = $files[0];		
    }
    return $result;
}

/**
 * 
 * Returns file data from table
 * @param $clickbank click bank file code
 */
function file_commerce_pro_load_file_by_clickbank_id_from_db($clickbank) {
    global $wpdb;
    $result = false;
    if (is_string($clickbank))
    {
    	$query = "select * from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_FILES . " where clickbank = " . $clickbank;
		$files = $wpdb->get_results($query);
		$result = $files[0];
    }
    return $result;
}

/**
 * Delete file data from table
 * @param $id
 */
function file_commerce_pro_delete_file_from_db($id) {
    global $wpdb;    
    if (is_numeric($id))
    {
    	$query = "delete from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_FILES . " where id = " . $id;
		$r = $wpdb->query($query);
		if (!$r) return false;			
    }
    return true;
}

/**
 * Update file status value (0 - not active, 1 - active)
 * @param $id
 * @param $value
 */
function file_commerce_pro_update_file_status($id,$value) {
    global $wpdb;    
    if (is_numeric($id))
    {
    	$query = $wpdb->prepare("update " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_FILES . " set status = %d where id = %d", array( $value, $id ));
		$r = $wpdb->query($query);
		if ($r === false) return false;
    }
    return true;
}

/**
 * Add a +1 to file hits value
 * @param $id file id
 */
function file_commerce_pro_update_file_hits($id) {
    global $wpdb;    
    if (is_numeric($id))
    {
    	$r = $wpdb->query("update " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_FILES . " set hits = hits+1 where id = " . $id);
		if ($r === false) return false;
    }
    return true;
}

/**
 * Returns an array of files (key => value) a user can download  
 * @param $uid user id
 */
function file_commerce_pro_get_available_files_for_user($uid) {
	global $wpdb;
	
    $result = false;
    if (is_numeric($uid))
    {
    	$query = $wpdb->prepare("select * from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS . " c, " . 
    				$wpdb->prefix . FILECOMMERCEPRO_TABLE_FILES . " f where c.fileid = f.id and and f.status > %d 
    				and c.userid = %d order by c.expire desc", array(0, $uid));
		$files = $wpdb->get_results($query);
		$result = $files;
    }
    return $result;
}

/**
 * Returns file name without dangerous chars
 * @param $filename
 */
function file_commerce_pro_sanitize_filename($filename) {
	$filename = str_replace(' ', '', $filename);
	
	return $filename;
}

/**
 * Returns complete url of a plugin script for admin area
 * @param $script script name (without trailing slashes)
 */
function file_commerce_pro_get_plugin_admin_url($script)  {
	return get_admin_url().'admin.php?page=file-commerce-pro/'.$script;
}

/**
 * 
 * Load a contract
 * @param $id
 */
function file_commerce_pro_load_contract($id) {
	global $wpdb; 
	
	if (is_numeric($id))
	{
		return $wpdb->get_row("SELECT * FROM " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS . " WHERE id = " . $id);	
	}
	return false;	
}

/**
 * 
 * Load a contract by userid and fileid
 * @param $userid
 * @param $fileid
 */
function file_commerce_pro_load_contract_by_user_and_file($userid,$fileid) {
	global $wpdb; 
	
	if (is_numeric($userid) && is_numeric($fileid))
	{
		return $wpdb->get_row("SELECT * FROM " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS . " WHERE userid = " . $userid . " AND fileid = " . $fileid);	
	}
	return false;	
}

/**
 * 
 * Load all contracts available for a user with many conditions
 * @param $userid
 * @param $fileid
 * @param $status
 * @param $minexpire
 */
function file_commerce_pro_load_contracts_by_data($userid,$fileid,$status,$minexpire) {
	global $wpdb;
	$result = false; 	
	if (is_numeric($userid) && is_numeric($fileid) && is_numeric($status) && is_numeric($minexpire))
	{
		$result = $wpdb->get_results("select * from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS .
        	" where fileid = " . $fileid . " and userid = " . $userid . " and status = " .
        	$status . " and expire > " . $minexpire . " order by expire desc");	
	}
	return $result;	
}

/**
 * Delete a contract from database 
 * @param $id
 */
function file_commerce_pro_delete_contract_from_db($id) {
	global $wpdb;    
    if (is_numeric($id))
    {
    	$query = "delete from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS . " where id = " . $id;
		$r = $wpdb->query($query);
		if (!$r) return false;
    }
    else
    {
    	return false;
    }
    return true;
}

/**
 * 
 * Save a contract
 * @param $userid
 * @param $fileid
 * @param $status
 * @param $expire expiration date in UNIX timestamp
 * @param $txnid transaction id, if available 
 * @param $gateway payment gateway
 */
function file_commerce_pro_save_contract($userid,$fileid,$status,$expire,$txnid,$gateway) {
	global $wpdb;		
	
	$contract = array();    
    $contract['userid'] = $userid;
    $contract['fileid'] = $fileid;
    $contract['status'] = $status;
    $contract['expire'] = $expire;
    $contract['txnid'] = $txnid;
    $contract['gateway'] = $gateway;    
    
    return $wpdb->insert($wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS, $contract, array('%d','%d','%d','%d','%s','%s'));
}

/**
 * 
 * Update contract row
 * @param $id
 * @param $update_array key => value array with fields to be updated
 */
function file_commerce_pro_update_contract($id,$update_array) {
	global $wpdb;		
	if (is_numeric($id))
	{
		return $wpdb->update($wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS, $update_array, array('id'=>$id));	
	}
    return false;
}

/**
 * 
 * Update contract row by file id
 * @param unknown_type $fileid
 * @param unknown_type $gateway
 * @param unknown_type $update_array
 */
function file_commerce_pro_update_contract_by_fileid($fileid,$update_array) {
	global $wpdb;		
	if (is_numeric($fileid))
	{
		return $wpdb->update($wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS, $update_array, array('fileid'=>$fileid));	
	}
    return false;
}

/**
 * Save ipn log
 * @param $userid
 * @param $fileid
 * @param $gateway
 * @param $data
 * @param $ipnval
 * @param $request
 */
function file_commerce_pro_save_ipn($userid,$fileid,$gateway,$data,$ipnval='',$request='') {
	global $wpdb; 
	
	$ipn = array();
    $ipn['fileid'] = $fileid;
    $ipn['userid'] = $userid;
    $ipn['gateway'] = $gateway;
    $ipn['data'] = $data;
    $ipn['created'] = time();
    $ipn['ipn'] = $ipnval;
    if (empty($request)) $ipn['request'] = serialize($_SERVER);
    else $ipn['request'] = $request;
    
    return $wpdb->insert($wpdb->prefix . FILECOMMERCEPRO_TABLE_IPN, $ipn, array('%d','%d','%s','%s','%d','%s','%s'));
}

/**
 * Load ipn log 
 * @param $id
 * @return ipn row as object, false in case of error
 */
function file_commerce_pro_load_ipn($id) {
	global $wpdb; 
	
	if (is_numeric($id))
	{
		return $wpdb->get_row("SELECT * FROM " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_IPN . " WHERE id = " . $id);	
	}
	return false;	
}

/**
 * 
 * Update ipn row
 * @param $id
 * @param $update_array key => value array with fields to be updated
 */
function file_commerce_pro_update_ipn($id,$update_array) {
	global $wpdb;		
	if (is_numeric($id))
	{
		return $wpdb->update($wpdb->prefix . FILECOMMERCEPRO_TABLE_IPN, $update_array, array('id'=>$id));	
	}
	return false;    
}

// REPORTS

/**
 * 
 * Returns valid contracts stipulated in the last 12 months, for reports
 */
function file_commerce_pro_repo_get_last_12_months_contracts() {
	global $wpdb;
		
	$query = "select concat(year(created), '-', date_format(created, '%m')) as d, count(id) as c
		from ". $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS ." where status='".FileCommercePro::CONTRACT_VALID."' group by concat(year(created), '-', date_format(created, '%m')) 
		order by d desc limit 12";	
	$res = $wpdb->get_results($query,'ARRAY_A');
	
	return $res; 
}

/**
 * 
 * Returns file hits, for reports
 */
function file_commerce_pro_repo_get_download_hits() {
	global $wpdb;	
	
	$query = "select title, hits from ". $wpdb->prefix . FILECOMMERCEPRO_TABLE_FILES ." where hits > 0 order by hits desc";	
	$res = $wpdb->get_results($query,'ARRAY_A');
	
	return $res;
}
