<?php

/**
 * Questo è il "listener" per le chiamate server to server di Plimus e viene chiamato
 * direttamente.
 * 
 * TODO: nella verifica della chiamata, quando il prezzo del file è diretto, lo stesso
 * viene controllato solo se è una notifica di pagamento completato in quanto non so se
 * viene passato anche in caso di refund.
 * 
 * TODO: gestire i ricorrenti (anche se dovrebbe già funzionare) ma occhio alle problematiche
 * del prezzo inserito direttamente: se viene chiamato un pagamento ricorrente con un prezzo
 * vecchio fallirebbe. La soluzione è mettere il prezzo pagato dentro il contratto iniziale
 * e recuperarlo dai dati del pagamento ricorrente, un casino.
 */

include '../../../wp-load.php';

/**
 * Verifica la chiamata in vari passi:
 * 
 * - chiama il sistema di verifica di PayPal
 * - verifica se il file esiste
 * - in caso di prezzo diretto, lo controlla
 * - verifica la corrispondenza con l'indirizzo email del merchant
 * 
 * Se qualcosa va male manda una notifica all'amministratore.
 */
function file_commerce_pro_paypal_verify($file) {
    global $file_commerce_pro;

    // Composes the request for PayPal verification
    $post_body = 'cmd=_notify-validate&';
    foreach ($_POST as $name => $value) {
        $post_body .= $name . '=' . urlencode(stripslashes($value)) . '&';
    }
    $post_body = substr($post_body, 0, -1);

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $file_commerce_pro->paypal_url);
    curl_setopt($ch, CURLOPT_VERBOSE, 1);

    //turning off the server and peer verification(TrustManager Concept).
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);

    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_POST, 1);

    //setting the nvpreq as POST FIELD to curl
    curl_setopt($ch, CURLOPT_POSTFIELDS, $post_body);

    //getting response from server
    $response = curl_exec($ch);
    $file_commerce_pro->log_debug($response);
    if ($response !== 'VERIFIED') {
        $file_commerce_pro->admin_mail('PayPal IPN not verified!', "The IPN below was not verified, try to resend it from PayPal IPN history panel.\n\n" . print_r($_POST, true));
        return null;
    }
    
    if (strpos(strtolower($file_commerce_pro->get_option('paypal_email')), strtolower($_POST['receiver_email'])) === false) {
        $file_commerce_pro->log_fatal('Wrong merchant email');
        $file_commerce_pro->admin_mail('PayPal IPN not verified!', "The IPN was repolrting an unknown merchant email\n\n" . print_r($_POST, true));
        die();
    }
    

    $file = $file_commerce_pro->get_file($_POST['item_number']);
    if (empty($file)) {
        $file_commerce_pro->log_fatal('File not found');
        $file_commerce_pro->admin_mail('PayPal IPN not verified!', "The IPN was referring a file not existing: " . $_POST['item_number'] . "\n\n" . print_r($_POST, true));
        return false;
    }

    if ($_POST['payment_status'] == 'Completed' && !empty($file->amount)) {
        if ($file->amount != $_POST['mc_gross'] || $file->currency != $_POST['mc_currency']) {
            $file_commerce_pro->log_fatal('Wrong amount or currency');
            $file_commerce_pro->admin_mail('PayPal IPN not verified!', "The IPN has the amount or the currency wrong\n\n" . print_r($_POST, true));
            return null;
        }
    }

    return $file;
}

$file_commerce_pro->log_info('paypal.php');
$file_commerce_pro->log_debug($_GET);
$file_commerce_pro->log_debug($_POST);

$file = file_commerce_pro_paypal_verify();

if ($file == null) {
    $file_commerce_pro->log_error('Paypal IPN unverified');
    die();
}


// Prepare the ipn data
$ipn = array();
$ipn['gateway'] = FileCommercePro::GATEWAY_PAYPAL;
$ipn['data'] = serialize($_POST);
$ipn['request'] = serialize($_SERVER);
$ipn['created'] = time();

switch ($_POST['payment_status']) {
    case 'Completed':
        $txnid = $_POST['txn_id'];
        $contract = $wpdb->get_row($wpdb->prepare("select * from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS . " where gateway='" . FileCommercePro::GATEWAY_PAYPAL . "' and txnid=%s", $txnid), ARRAY_A);

        if (!empty($contract) && $contract['status'] == FileCommercePro::CONTRACT_VALID) {
            $file_commerce_pro->log_error('Duplicated IPN, contract already in paid status');
            die();
        }

        // There is no contract for this transaction
        if (empty($contract)) {
            // No user associated, create a new account
            if (empty($_POST['custom'])) {
                $user = $file_commerce_pro->create_user($_POST['payer_email']);
            } else {
                $user = get_user_by('id', $_POST['custom']);
            }

            $contract = array();
            $contract['fileid'] = $_POST['item_number'];
            $contract['userid'] = $user->ID;
            $contract['status'] = FileCommercePro::CONTRACT_INVALID;
            $contract['gateway'] = FileCommercePro::GATEWAY_PAYPAL;
            $contract['txnid'] = $txnid;
            $contract['amount'] = $_POST['mc_gross'] * 100;
            $contract['net_amount'] = ($_POST['mc_gross'] - $_POST['mc_fee']) * 100;
            $contract['name'] = stripslashes($_POST['address_name']);
            $contract['email'] = $_POST['payer_email'];
            $contract['token'] = md5(time() . rand());
        }

        $contract['expire'] = $file_commerce_pro->get_contract_expiration_value($file->duration);
        $contract['status'] = FileCommercePro::CONTRACT_VALID;

        if (empty($contract['id'])) {
            //$contract = apply_filters('file_commerce_pro_contract', $contract);
            $php_code = $file_commerce_pro->get_option('extend_contract');
            if (!empty($php_code))
                eval($php_code);
            $wpdb->insert($wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS, $contract);
            $contract['id'] = $wpdb->insert_id;
        }
        else {
            // Should not the case
            $id = $contract['id'];
            unset($contract['id']);
            $wpdb->update($wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS, $contract, array('id' => $id));
            $contract['id'] = $id;
        }

        $ipn['contractid'] = $contract['id'];
        $wpdb->insert($wpdb->prefix . FILECOMMERCEPRO_TABLE_IPN, $ipn);

        $file_commerce_pro->notify_new_contract($contract, $user);

        die();
        break;

    case 'Reversed':
    case 'Refunded':
        $txnid = $_POST['parent_txn_id'];
        $contract = $wpdb->get_row($wpdb->prepare("select * from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS . " where gateway='" . FileCommercePro::GATEWAY_PAYPAL . "' and txnid=%s", $txnid), ARRAY_A);
        if (empty($contract)) {
            $file_commerce_pro->log_error('Contract not found.');
            $wpdb->insert($wpdb->prefix . FILECOMMERCEPRO_TABLE_IPN, $ipn);
            die();
        }

        $ipn['contractid'] = $contract['id'];
        $wpdb->insert($wpdb->prefix . FILECOMMERCEPRO_TABLE_IPN, $ipn);

        $wpdb->update($wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS, array('status' => FileCommercePro::CONTRACT_INVALID), array('id' => $contract['id']));
        $file_commerce_pro->notify_invalidated_contract($contract);
        die();
        break;

    case 'Failed':
        break;

    case 'Canceled_Reversal':
        $txnid = $_POST['parent_txn_id'];
        $contract = $wpdb->get_row($wpdb->prepare("select * from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS . " where gateway='" . FileCommercePro::GATEWAY_PAYPAL . "' and txnid=%s", $txnid), ARRAY_A);
        if (empty($contract)) {
            $file_commerce_pro->log_error('Contract not found.');
            $wpdb->insert($wpdb->prefix . FILECOMMERCEPRO_TABLE_IPN, $ipn);
            die();
        }

        $ipn['contractid'] = $contract['id'];
        $wpdb->insert($wpdb->prefix . FILECOMMERCEPRO_TABLE_IPN, $ipn);

        $wpdb->update($wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS, array('status' => FileCommercePro::CONTRACT_VALID), array('id' => $contract['id']));
        die();
        break;

    case 'Created':
    case 'Denied': // Prevously pending
    case 'Expired': // ???
    case 'Processed':
    case 'Voided':
        break;
}
die();
?>
