<?php

/*
  Plugin Name: File Commerce Pro
  Plugin URI: http://www.satollo.net/plugins/file-commerce
  Description: Sell file online quickly and simply.
  Version: 2.1.0.2
  Author: Satollo
  Author URI: http://www.satollo.net
  Disclaimer: Use at your own risk. No warranty expressed or implied is provided.
 */

define('FILE_COMMERCE_PRO_VERSION', '2.1.0.2');

@include_once(dirname(__FILE__) . '/commons.php');

$file_commerce_pro = new FileCommercePro();

class FileCommercePro {

    var $title = 'File Commerce Pro';
    var $uid; // Will be the plugin folder name and used as key for plugin options
    var $options = null;
    var $time_limit = 0;
    var $option_name_clickbank_vendor_code = 'clickbank_vendor_code';
    var $option_name_clickbank_secret_key = 'clickbank_secret_key';
    //GET params
    var $param_plugin = 'file-commerce-pro';
    var $param_action_download = 'download';
    // Click bank related
    var $param_action_buy_clickbank = 'buy_clickbank';
    var $param_action_clickbank_notification = 'clickbanknotification';
    // Paypal related
    var $paypal_url = 'https://www.paypal.com/cgi-bin/webscr';
    //var $paypal_url = 'https://www.sandbox.paypal.com/cgi-bin/webscr';

    var $secret;

    //Gateway value
    const GATEWAY_CLICKBANK = 'clickbank';
    const GATEWAY_PAYPAL = 'paypal';
    const GATEWAY_PLIMUS = 'plimus';


    // Contract status
    const CONTRACT_INVALID = 0; // This contract is not effective
    const CONTRACT_VALID = 1; // This status means the contract has been paid (it can be expired, thought)
    // File status
    const FILE_DISABLED = 0;
    const FILE_ON_SALE = 1;
    const FILE_ONLY_DOWNLOAD = 2;
    const FILE_FREE_REGISTERED = 3;
    const FILE_FREE = 4;
    const FILE_FREE_WITH_CONTRACT = 5; // not implemented for who have a contract active
    // File free download
    const DOWNLOAD_IS_NOT_FREE = 0;
    const DOWNLOAD_IS_FREE_FOR_REGISTERED_USERS = 1;
    const DOWNLOAD_IS_FREE_FOR_ALL_USERS = 2;

    //const PAYPAL_TYPE_BUTTON = 0;
    //const PAYPAL_TYPE_FORM = 1;
    //const PAYPAL_TYPE_PRICE = 2;

    /** The constructor is called on this file load, so we keep it as light as possible. Other initializations
     * are delegated to "hook_init" called when WordPress initializes the plugins.
     */
    function __construct() {
        global $wpdb;

        // Unique string id for this plugin (its folder name... it's clearly unique)
        $this->uid = basename(dirname(__FILE__));

        // Tries to compute this session time limit, but some time has been already consumed before.
        // Time limit is the 90% of PHP max execution time.
        $max_time = (int) (ini_get('max_execution_time') * 0.9);
        if ($max_time == 0) $max_time = 3600;
        $this->time_limit = time() + $max_time;

        add_action('init', array(&$this, 'hook_init'));

        // Activation and deactivation hooks (cannot be placed on "hook_init", that
        // function is not called on activation)
        register_activation_hook(__FILE__, array(&$this, 'hook_activate'));
        register_deactivation_hook(__FILE__, array(&$this, 'hook_deactivate'));
    }

    function hook_init() {
        $this->secret = get_option($this->uid . '-secret');
        
        if (get_option('file-commerce-pro-version') != FILE_COMMERCE_PRO_VERSION) {
          $this->log_fatal('Version change without deactivation from ' . get_option('file-commerce-pro-version') . ' to ' . FILE_COMMERCE_PRO_VERSION);
          $this->hook_activate();
        }        

        add_action('admin_menu', array(&$this, 'hook_admin_menu'));
        add_action('admin_head', array(&$this, 'hook_admin_head'));
        add_action('register_form', array(&$this, 'hook_register_form'));
        add_action('wp_head', array(&$this, 'hook_wp_head'));

        //ajax callbacks
        if (is_admin()) {
            add_action('wp_ajax_filecommercepro_admin_action', array(&$this, 'admin_ajax_action_callback'));
            if (strpos($_GET['page'], 'file-commerce-') === 0) {
                wp_enqueue_script('jquery-ui-tabs');
                wp_enqueue_script('media-upload');
                wp_enqueue_script('thickbox');
                wp_enqueue_style('thickbox');
            }
        }

        add_filter('login_message', array(&$this, 'hook_login_message'));
        add_shortcode('file_commerce_pro', array(&$this, 'shortcode_file_commerce_pro'));
        add_shortcode('file_commerce_pro_support', array(&$this, 'shortcode_file_commerce_pro_support'));
        add_shortcode('file_commerce_pro_lock', array(&$this, 'shortcode_file_commerce_pro_lock'));
    }

    function hook_login_message($message) {
        return $message . $this->get_option('login_message');
    }

    /**
     * Adds a message on botton of WordPress registration form.
     */
    function hook_register_form() {
        echo '<p>';
        echo $this->get_option('register_message');
        echo '</p>';
    }

    function shortcode_file_commerce_pro_support($attrs, $content) {
        global $wpdb, $current_user, $cache_stop;
        
        $cache_stop = true;

        get_currentuserinfo();

        $buffer = '<div class="file-commerce-pro-support">';

        if (!is_user_logged_in()) {
            $buffer .= '<div class="file-commerce-pro-support-login">';
            $buffer .= $this->replace($this->get_option('support_login'));
            $buffer .= '<form action="';
            $buffer .= wp_login_url($_SERVER['REQUEST_URI']);
            $buffer .= '" method="post">';
            $buffer .= __('Username');
            $buffer .= ':<br /><input type="text" size="20" name="log"/><br />';
            $buffer .= __('Password');
            $buffer .= ':<br /><input type="password" size="20" name="pwd"/><br />';
            $buffer .= '<input type="submit" value="';
            $buffer .= esc_attr__('Log In');
            $buffer .= '" name="wp-submit">';
            $buffer .= '<input type="hidden" value="forever" name="rememberme"/>';
            $buffer .= '</form><br />';
            $buffer .= '<a href="' . site_url('wp-login.php?action=lostpassword', 'login') . '">';
            $buffer .= __('Lost your password?');
            $buffer .= '</a>';
            $buffer .= '</div></div>';
            return $buffer;
        }

        if (isset($_POST['fcp_message'])) {

            $unique = md5($time);

            $headers = '';
            $headers .= "Content-type: text/plain; charset=UTF-8\n";
            //$headers .= "Reply-To: " . $current_user->user_email . "\n";
            $headers .= "From: " . $current_user->user_email . " <" . $current_user->user_email . ">\n";

            $message .= "Login: " . $current_user->user_login . "\n";
            $message .= "Email: " . $current_user->user_email . "\n";

            if ($_POST['fcp_file'] == 0) {
                $message .= "GENERIC REQUEST\n";
            } else {
                $file = file_commerce_pro_load_file_from_db($_POST['fcp_file']);
                $message .= "File: " . strip_tags($file->title) . "\n";

                $contracts = $this->get_active_contracts($current_user->ID, $file->id);
                if (empty($contracts))
                        $message .= "Contract: no active contract found\n";
                else
                        $message .= "Contract: at least one active contract found\n";
            }
            $message .= "\n";

            $message .= "--------------------------------------------------\n";
            $message .= trim(stripslashes($_POST['fcp_message'])) . "\n\n";
            $message .= "--------------------------------------------------\n";
            $message .= "Yours, File Commerce Pro";

            if (!empty($file->support_email)) {
                $r = $this->mail($file->support_email, '[Support Request] from ' . $current_user->user_email, $message, $headers);
                $r = $this->mail($this->get_option('support_email'), '[Support Request Copy] from ' . $current_user->user_email, $message, $headers);
            } else {
                $r = $this->mail($this->get_option('support_email'), '[Support Request] from ' . $current_user->user_email, $message, $headers);
            }

            $buffer .= '<div class="file-commerce-pro-support-thankyou">';
            if (!$r)
                    $buffer .= "<p><strong>Unable to send the message, sorry!</strong></p>";
            else $buffer .= $this->get_option('support_ok');
            $buffer .= '</div></div>';
            return $buffer;
        }

        $buffer .= '<form method="post" action="" enctype="multipart/form-data">';

        // File list
        $files = $wpdb->get_results("select id, title from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_FILES . " where status<>" .
                self::FILE_DISABLED . " order by title");
        $buffer .= '<p><select name="fcp_file">';
        foreach ($files as $file) {
            $contracts = $this->get_active_contracts($current_user->ID, $file->id);
            if (empty($contracts)) continue;
            $buffer .= '<option value="' . $file->id . '">' . strip_tags($file->title) . '</option>';
        }
        if ($this->get_option('support_generic') == 1) {
          $buffer .= '<option value="0">' . $this->get_option('support_generic_label', 'Generic request', true) . '</option>';
        }
        $buffer .= '</select></p>';

        $buffer .= '
            <p>
            <textarea cols="70" rows="15" name="fcp_message"></textarea>
            </p>
            <!--
            <p>Immagine:<br />
            <input type="file" name="ma_image"/> (obbligatoria di tipo jpg, gif o png)
            </p>
            -->
            ';


        $buffer .= '<p>
            <input type="submit"/>
            </p>
            </form>
            ';

        $buffer .= '</div>';
        return $buffer;
    }

    function shortcode_file_commerce_pro($attrs, $content) {
        global $cache_stop;
        
        $cache_stop = true;

        // Show the single sale box for a single file (usually on the single file pitch page)
        if (isset($attrs['file'])) {
            return $this->do_list_available_file($attrs['file'], null, $attrs['exclude'], $attrs['type']);
        }

        if (isset($attrs['parent'])) {
            return $this->do_list_available_file(null, $attrs['parent'], $attrs['exclude'], $attrs['type']);
        }

        $content .= $this->do_list_available_file(null, null, $attrs['exclude'], $attrs['type']);

        return $content;
    }

    function shortcode_file_commerce_pro_lock($attrs, $content) {
        global $cache_stop, $wpdb;

        $cache_stop = true;

        // Not logged in... show the lock message
        if (!is_user_logged_in()) {
            return $this->get_option('lock_message');
        }

        // For every users
        if (empty($attrs['file'])) {
            return do_shortcode($content);
        }

        // The user must have at least one contract active
        if (trim($attrs['file']) == '*') {
            $r = $wpdb->get_var("select count(*) from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS .
                    " where userid = " . get_current_user_id() . " and status = " . FileCommercePro::CONTRACT_VALID .
                    " and expire > " . time());
            if ($r > 0) return do_shortcode($content);
            else return $this->get_option('lock_message');
        }

        $ids = explode(',', $attrs['file']);
        $r = $wpdb->get_var("select count(*) from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS .
                " where userid = " . get_current_user_id() . " and status = " . FileCommercePro::CONTRACT_VALID .
                " and fileid in (" . $attrs['file'] . ") and expire > " . time());
        if ($r > 0) return do_shortcode($content);
        else return $this->get_option('lock_message');
    }

    function is_downloadable($file) {
        global $current_user, $wpdb;

        // Active contracts
        $contracts = file_commerce_pro_load_contracts_by_data($current_user->ID, $file->id, FileCommercePro::CONTRACT_VALID, time());

        $has_contracts = !empty($contracts);

        // Search for a parent file VALID contract when the file has one or more parents files. The search take place only
        // if we have not an active contract, otherwise the file is already available to the customer.
        if (empty($contracts) && $file->parent_fileid != '') {
            $parent_contracts = $wpdb->get_results("select * from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS .
                    " where fileid in (" . $file->parent_fileid . ") and userid = " . $current_user->ID . " and status = " .
                    FileCommercePro::CONTRACT_VALID . " and expire > " . time() . " order by expire desc");
            $has_contracts = !empty($parent_contracts);
        }

        $verified = false;
        switch ($file->status) {
            // When on sale or downloadable with an active contract
            case FileCommercePro::FILE_ON_SALE:
            case FileCommercePro::FILE_ONLY_DOWNLOAD:
                $verified = $has_contracts;
                break;

            // When downloadable to all registered users
            case FileCommercePro::FILE_FREE_REGISTERED:
                $verified = is_user_logged_in();
                break;

            // When downloadable when a user has at least on active contract with us
            case FileCommercePro::FILE_FREE_WITH_CONTRACT:
                $r = $wpdb->get_var("select count(*) from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS .
                        " where userid = " . get_current_user_id() . " and status = " . FileCommercePro::CONTRACT_VALID .
                        " and expire > " . time());
                $verified = $r > 0;
                break;

            // Free for all!
            case FileCommercePro::FILE_FREE:
                $verified = true;
                break;
        }
        return $verified;
    }

    function is_virtual($file) {
        return empty($file->filename) && ($file->linked_fileid == 0);
    }

    /**
     *
     * @global type $wpdb
     * @global type $current_user
     * @global type $contract
     * @param type $file_ids Comma separated list of file IDs to show, parent IDs is ignored is this parameter is used
     * @param type $parent_ids Comma separated list if file IDs used as "parent filter" (only files with one of those parents is shown)
     * @param type $exclude_ids Comma separated list if file IDs to be filtered out from the list
     * @param type $type Type of list: "null" for a standard list, "contracts" for a user's contract list by file
     * @return type 
     */
    function do_list_available_file($file_ids = null, $parent_ids = null, $exclude_ids = null, $type=null) {
        global $wpdb, $current_user;

        // Exclude disabled files, always.
        $query = "select * from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_FILES . " where status<>" .
                self::FILE_DISABLED . " order by idx";

        // Loads all the files
        $all_files = $wpdb->get_results($query);
        $files = array();

        $file_ids = empty($file_ids) ? null : explode(',', $file_ids);
        $exclude_ids = empty($exclude_ids) ? null : explode(',', $exclude_ids);
        $parent_ids = empty($parent_ids) ? null : explode(',', $parent_ids);
        foreach ($all_files as &$file) {
            // If explicitely excluded, skip it
            if (!empty($exclude_ids)) {
                if (array_search($file->id, $exclude_ids) !== false) continue;
            }

            // No other filter
            if (empty($parent_ids) && empty($file_ids)) {
                $files[] = $file;
                continue;
            }

            // If there is a parent and the file is compatible
            if (!empty($parent_ids)) {
                if (!empty($file->parent_fileid)) {
                    $file_parent_ids = explode(',', $file->parent_fileid);
                    $intersect = array_intersect($parent_ids, $file_parent_ids);
                    if (!empty($intersect)) {
                        $files[] = $file;
                        continue;
                    }
                }
            }

            // If the file is explicitely included
            if (!empty($file_ids)) {
                if (array_search($file->id, $file_ids) !== false) {
                    $files[] = $file;
                    continue;
                }
            }
        }

        $buffer = '<div class="file-commerce-pro-files file-commerce-pro-files-' . $type . '">';

        foreach ($files as &$file) {

            $is_downloadable = $this->is_downloadable($file);

            $buffer .= '<div class="file-commerce-pro-file file-commerce-pro-file-' . $file->id . ($is_downloadable ? ' file-commerce-pro-file-active' : '') . '" id="file-commerce-pro-file-' . $file->id . '">';

            // The title
            if (!empty($file->title)) {
                $buffer .= '<div class="file-commerce-pro-title">';
                if (empty($file->url)) $buffer .= $file->title;
                else
                        $buffer .= '<a href="' . $file->url . '">' . $file->title . '</a>';
                $buffer .= '</div>';
            }
            if (!empty($file->image)) {
                $buffer .= '<div class="file-commerce-pro-image"><img src="' . $file->image . '"/></div>';
            }

            if (!empty($file->description)) {
                $buffer .= '<div class="file-commerce-pro-description">' . $file->description . '</div>';
            }

            if (!empty($file->price)) {
                $buffer .= '<div class="file-commerce-pro-price">' . $file->price . '</div>';
            }

            // Download link or purchased note
            if ($is_downloadable) {
                // If there is neither a direct physical file nor a reference to another file we show the "purchsed" label
                if ($this->is_virtual($file)) {
                    $buffer .= '<div class="file-commerce-pro-purchased">' . $this->get_option('purchased_label', 'PURCHASED') . '</div>';
                } else {
                    $buffer .= '<div class="file-commerce-pro-download">';
                    $buffer .= '<a href="' . plugins_url('download.php', __FILE__) . '?f=' . $file->id . '">' . $this->get_option('download_label') . '</a>';
                    $buffer .= '</div>';
                }
            } else {
                if (!is_user_logged_in() && $this->get_option('login') == 1) {
                    $buffer .= '<div class="file-commerce-pro-login"><a href="' . wp_login_url(get_permalink()) . '">' . $this->get_option('login_label') . '</a></div>';
                } else {
                    // Payment button only for on sale files!
                    if ($file->status == FileCommercePro::FILE_ON_SALE) {
                        $buffer .= '<div class="file-commerce-pro-buttons">';
                        // Add pay now buttons
                        $buffer .= $this->paypal_button($file);
                        if ($file->plimus != '' && $file->plimus[0] != '-')
                                $buffer .= $this->plimus_button($file);
                        
                        $buffer .= $this->clickbank_button($file);

                        $buffer .= '</div>';
                        $buffer .= '<div class="file-commerce-pro-buttons-note">';
                        $buffer .= $this->get_option('buttons_note');
                        $buffer .= '</div>';
                    }
                }
            }

            $contracts = $wpdb->get_results("select * from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS .
              " where fileid = " . $file->id . " and userid = " . $current_user->ID . " and status = " .
              self::CONTRACT_VALID . " order by expire desc");	

            if (!empty($contracts)) {
                $buffer .= '<div class="file-commerce-pro-custom">';
                $php_code = $this->get_option('extend_custom_display');
                foreach ($contracts as $contract) {
                    //if (empty($contract->custom)) continue;
                    if (empty($php_code))
                            $buffer .= '<div>' . $contract->custom . '</div>';
                    else {
                        $custom = $contract->custom;
                        $buffer .= '<div>' . eval($php_code) . '</div>';
                    }
                }
                $buffer .= '</div>';
            }

            // TODO: If the file has direct contracts show them in some way, for example the
            // expire date.
            if (!empty($contracts)) {
                $buffer .= '<div class="file-commerce-pro-contracts">';
                $buffer .= '<div>' . $this->get_option('contracts_label') . '</div>';
                foreach ($contracts as &$contract) {
                    $buffer .= '<p>Contract ' . $contract->id . ' expires on ' . date('Y-m-d', $contract->expire) . '</p>';
                }
                $buffer .= '</div>';
            }

            $buffer .= '<div style="clear: both"></div></div>';
        }
        return $buffer . '</div>';
    }

    function hook_admin_menu() {
        // Indipendent menu block
        add_menu_page('File Commerce', 'File Commerce', 10, $this->uid . '/main.php', '', '');
        add_submenu_page($this->uid . '/main.php', 'Configuration', 'Configuration', 10, $this->uid . '/main.php');
        add_submenu_page($this->uid . '/main.php', 'Files', 'Files', 10, $this->uid . '/repository.php');
        //add_submenu_page($this->uid . '/repository.php', 'Add File', 'Add File', 10, $this->uid . '/repository_add.php');
        add_submenu_page($this->uid . '/repository.php', 'Edit File', 'Edit File', 10, $this->uid . '/repository_edit.php');

        // Contracts management
        add_submenu_page($this->uid . '/main.php', 'Contracts', 'Contracts', 10, $this->uid . '/contracts.php');
        add_submenu_page($this->uid . '/contracts.php', 'New contract', 'New contract', 10, $this->uid . '/contracts_add.php');
        add_submenu_page($this->uid . '/contracts.php', 'Edit contract', 'Edit contract', 10, $this->uid . '/contracts_edit.php');

        add_submenu_page($this->uid . '/main.php', 'Mailing', 'Mailing', 10, $this->uid . '/mailing.php');
        add_submenu_page($this->uid . '/main.php', 'User Guide', 'User Guide', 10, $this->uid . '/user_guide.php');
        add_submenu_page($this->uid . '/main.php', 'Reports', 'Reports', 10, $this->uid . '/reports.php');
    }

    function hook_admin_head() {
        //load static resources only on plugin admin pages
        if (strpos($_SERVER['QUERY_STRING'], 'file-commerce-pro') !== false) {
            echo '<script type="text/javascript" src="' . plugins_url('/js/admin.js', __FILE__) . '"></script>';
            echo '<link rel="stylesheet" href="' . plugins_url('/css/admin.css', __FILE__) . '" type="text/css" media="all" />';
            echo '<link rel="stylesheet" href="' . plugins_url('/css/jquery-ui.css', __FILE__) . '" type="text/css" media="all" />';
            //load google api for charts
            if (strpos($_SERVER['QUERY_STRING'], 'reports.php') !== false) {
                echo '<script type="text/javascript" src="http://www.google.com/jsapi"></script>';
                echo '<script type="text/javascript">google.load(\'visualization\', \'1\', {packages: [\'corechart\']});</script>';
                echo '<script type="text/javascript">google.setOnLoadCallback( filecommercepro_onloadcallback );</script>';
            }
        }
    }

    function log($text, $level) {
        if ($level != 0 && $this->get_option('log') < $level) return;
        //$db = debug_backtrace(false);
        $time = date('d-m-Y H:i:s ');
        switch ($level) {
            case 0: $time .= '- FATAL - ';
                break;
            case 1: $time .= '- ERROR - ';
                break;
            case 2: $time .= '- INFO  - ';
                break;
            case 3: $time .= '- DEBUG - ';
                break;
        }
        if (is_array($text) || is_object($text)) $text = print_r($text, true);
        file_put_contents(WP_CONTENT_DIR . '/logs/' . $this->uid . '-' . $this->secret . '.txt', $time . $text . "\n", FILE_APPEND | FILE_TEXT);
        //file_put_contents(dirname(__FILE__) . '/log.txt', $time . $db[1]['function'] . '() - ' . $text . "\n", FILE_APPEND | FILE_TEXT);
    }

    function log_fatal($text) {
        $this->log($text, 0);
    }

    function log_error($text) {
        $this->log($text, 1);
    }

    function log_info($text) {
        $this->log($text, 2);
    }

    function log_debug($text) {
        $this->log($text, 3);
    }

    /** Returns plugin default options. Modify here adding values or loading from file when language dependant. */
    function get_default_options() {
        $sitename = strtolower($_SERVER['SERVER_NAME']);
        if (substr($sitename, 0, 4) == 'www.') $sitename = substr($sitename, 4);
        $thankyou = "<h1>Thank you</h1>
<p>Thank you for your purchase it has been successfully registered.</p>
<p><strong>Payment notes</strong></p>
<p>If you paid with Click Bank, your credit card will report ClickBank or CLKBANK*COM not the product name. It's ok. and it's the way Click Bank work.</p>
<p><strong>How to download the file</strong></p>
<p>If you have not an account on this site. Your credentials <strong>has just been mailed to your payment mail box</strong>, check for them (even in the spam/unsolicited folder). It can take <strong>from 5 to 15 minutes</strong>, be patience. Then <a href='{login_url_download}'>log in here</a></p>
<p>If you already have an account, <a href='{login_url_download}'>log in here</a> to download your files (if you're already logged in you should have the file list already available below).</p>
<p><strong>Got problems or need to contact me?</strong></p>
<p>For any problems with this purchase, please contact me at " . get_option('admin_email') . "</p>
<p><strong>Important note</strong>: if you paid with eCheck or other non-instant payment methods, please be patience and your contract will be activated soon (it may require two or three days).</p>";

        // Messaggio di default di attivazione di un nuovo contratto.
        $notify_ok_subject = "{file_title} details and your account credentials";
        $notify_ok_body =
                "Thank you for your purchase of {file_title}.\n\n" .
                "It can now be downloaded following the link below\n\n" .
                "   {file_url}\n\n" .
                "and the access will be granted until\n\n" .
                "   {contract_expire}\n\n" .
                "You can even download the file logging in at this address:\n\n" .
                "   {login_url_download}\n\n" .
                "using your credentials below\n\n" .
                "   Username: {user_login}\n\n" .
                "If you forgot your password it can be recovered here:\n\n" .
                "   {password_url}\n\n" .
                "For any question or future reference, please keep this email or annotate the following numbers:\n\n" .
                "   Contract No.: {contract_id}\n" .
                "   User No.: {user_id}\n" .
                "   File No.: {file_id}\n\n" .
                "Have a nice day!";

        $notify_ko_subject = "Contract invalidated";
        $notify_ko_body = "Hi,\n\nThe contract '{contract_id}' for the file\n\n{file_title}\n\nhas been invalidated.\n\n" .
                "For any question, write us.\n\nHave a nice day.";

        $new_user_subject = "Account created on " . get_option('blogname') . " (the password is enclosed)";
        $new_user_body = "Hi,\n\nyour account has just been created. Use the credentials below to access the site and download your files.\n\n" .
                "login: {user_login}\n" .
                "password: {user_password}\n\n" .
                "You can login here: {login_url_download}\n\n" .
                "For any question, write us.\n\nHave a nice day.";

        $support_ok = '<p>Your request has been sent. Thank you!</p>';
        $support_login = '<p>You must log in to send support requests. <a href="{login_url_back}">Log in here</a>.</p>';

        $buttons_note = 'Clicking a payment button will send you on a SECURE payment gateway and then back here to download the file.';

        return array(
            'login' => 1,
            'sender_name' => get_option('blogname'),
            'sender_email' => 'wordpress@' . $sitename,
            'thankyou' => $thankyou,
            'notify_ok_subject' => $notify_ok_subject,
            'notify_ok_body' => $notify_ok_body,
            'notify_ko_subject' => $notify_ko_subject,
            'notify_ko_body' => $notify_ko_body,
            'login_label' => 'Log in or register',
            'download_label' => 'DOWNLOAD NOW!',
            'purchased_label' => 'DOWNLOAD NOW!',
            'support_ok' => $support_ok,
            'support_login' => $support_login,
            'support_email' => get_option('admin_email'),
            'new_user_subject' => $new_user_subject,
            'new_user_body' => $new_user_body,
            'log' => 1,
            'buttons_note' => $buttons_note
        );
    }

    /** Returns the plugin options. */
    function get_options($suffix = '') {
        // Non main options are loaded each time
        if ($suffix != '') return get_option($this->uid . $suffix, array());
        if ($this->options == null)
                $this->options = get_option($this->uid, array());
        return $this->options;
    }

    function set_options($options, $suffix = '') {
        update_option($this->uid . $suffix, $options);
        if ($suffix == '') $this->options = $options;
    }

    /** Returns a plugin option by name, only for main options */
    function get_option($name, $default=null, $empty_as_missing=false) {
        if ($this->options == null)
                $this->options = get_option($this->uid, array());
        $value = $this->options[$name];
        if ($empty_as_missing) {
          $value = trim($value);
          if (empty($value)) return $default;
        }
          
        return isset($value) ? $value : $default;
    }

    function hook_activate() {
        global $wpdb, $charset_collate;

        if (!wp_mkdir_p(WP_CONTENT_DIR . '/logs')) {
            // The log is sent to the PHP default log
            error_log('Unable to create the ' . WP_CONTENT_DIR . '/logs folder: logging won\'t work!');
        }

        if (!get_option($this->uid . '-secret'))
                update_option($this->uid . '-secret', md5(rand(0, time())));

        // OPTIONS //
        update_option($this->uid, array_merge($this->get_default_options(), get_option($this->uid, array())));
        add_option($this->uid . '_contracts', array(), null, 'no');

        // DATABASE //

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        $table_name = $wpdb->prefix . FILECOMMERCEPRO_TABLE_FILES;

        $sql = "CREATE TABLE " . $table_name . " (
            id mediumint(9) NOT NULL AUTO_INCREMENT,
            price varchar(255) NOT NULL DEFAULT '',
            duration int NOT NULL,
            filename tinytext NOT NULL,
            title varchar(255) NOT NULL,
            mime varchar(20) NOT NULL,
            paypal_enabled tinyint(2) NOT NULL DEFAULT 0,
            paypal_type tinyint(2) NOT NULL DEFAULT 0,    
            paypal varchar(255) NOT NULL DEFAULT '',
            paypal_form text,
            plimus_enabled tinyint(2) NOT NULL DEFAULT 0,
            plimus varchar(255) NOT NULL DEFAULT '',
            clickbank_enabled tinyint(2) NOT NULL DEFAULT 0,
            clickbank varchar(255) NOT NULL DEFAULT '',
            status tinyint(2) NOT NULL DEFAULT 1,
            image varchar(255) NOT NULL DEFAULT '',
            url varchar(255) NOT NULL DEFAULT '',
            description text,
            idx tinyint(2) NOT NULL DEFAULT 0,
            hits int(11) NOT NULL DEFAULT 0,
            linked_fileid mediumint(9) NOT NULL DEFAULT 0,
            parent_fileid varchar(255) NOT NULL DEFAULT '',
            support_email varchar(255) NOT NULL DEFAULT '',
            version varchar(20) NOT NULL DEFAULT '',  
            notify_ok_subject varchar(200) NOT NULL DEFAULT '',  
            notify_ok_body text, 
            amount varchar(20) NOT NULL DEFAULT '',  
            currency varchar(20) NOT NULL DEFAULT 'USD',  
            multicontract tinyint(2) NOT NULL DEFAULT 0,   
            PRIMARY KEY id (id)
            ) $charset_collate;";
        dbDelta($sql);
        $wpdb->query("alter table " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_FILES . " CONVERT TO CHARACTER SET " . $wpdb->charset);

        $table_name = $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS;
        $sql = "CREATE TABLE " . $table_name . " (
            id int(11) NOT NULL AUTO_INCREMENT,
            userid int(11) NOT NULL DEFAULT 0,
            affiliateid int(11) NOT NULL DEFAULT 0,
            fileid int(11) NOT NULL DEFAULT 0,
            status tinyint(4) NOT NULL DEFAULT 0,
            expire bigint(20) DEFAULT 0 NOT NULL,
            txnid varchar(50) NOT NULL DEFAULT '',
            gateway varchar(20) NOT NULL DEFAULT '',
            amount int(11) NOT NULL DEFAULT 0,
            net_amount int(11) NOT NULL DEFAULT 0,
            created timestamp DEFAULT CURRENT_TIMESTAMP,
            name varchar(100) NOT NULL DEFAULT '',
            email varchar(100) NOT NULL DEFAULT '',
            token varchar(50) NOT NULL DEFAULT '',
            custom text,
            PRIMARY KEY id (id)
            ) $charset_collate;";
        dbDelta($sql);
        $wpdb->query("alter table " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS . " CONVERT TO CHARACTER SET " . $wpdb->charset);

        $wpdb->query("update " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS . " set token=md5(rand()) where token=''");

        $table_name = $wpdb->prefix . FILECOMMERCEPRO_TABLE_DOWNLOADS;
        $sql = "CREATE TABLE " . $table_name . " (
            id int(11) NOT NULL AUTO_INCREMENT,
            userid int(11) NOT NULL DEFAULT 0,
            fileid int(11) NOT NULL DEFAULT 0,
            ip varchar(50) NOT NULL DEFAULT '',
            created bigint NOT NULL DEFAULT 0,
            PRIMARY KEY id (id)
            ) $charset_collate;";
        dbDelta($sql);
        $wpdb->query("alter table " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_DOWNLOADS . " CONVERT TO CHARACTER SET " . $wpdb->charset);

        $table_name = $wpdb->prefix . FILECOMMERCEPRO_TABLE_IPN;
        $sql = "CREATE TABLE " . $table_name . " (
            id mediumint(9) NOT NULL AUTO_INCREMENT,
            userid mediumint(9) NOT NULL default 0,
            fileid mediumint(9) NOT NULL default 0,
            contractid mediumint(9) NOT NULL default 0,
            gateway varchar(15) NOT NULL  default '',
            data text,
            created bigint(20) not null default 0,
            ipn varchar(20) not null default '',
            request text,
            PRIMARY KEY id (id)
            ) $charset_collate;";
        dbDelta($sql);
        $wpdb->query("alter table " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_IPN . " CONVERT TO CHARACTER SET " . $wpdb->charset);

        // CUSTOM UPLOAD DIR //
        //create upload custom dir into wp upload dir
        $custom_upload_dir = $this->get_wp_upload_dir_path() . DIRECTORY_SEPARATOR . FILECOMMERCEPRO_FILE_UPLOAD_DIR;
        if (!file_exists($custom_upload_dir)) {
            if (!mkdir($custom_upload_dir)) {
                
            }
        }
        //copy some useful files into custom upload dir to prevent direct access to folder
        @copy(dirname(__FILE__) . '/files/index.php', $custom_upload_dir . 'index.php');
        @copy(dirname(__FILE__) . '/files/.htaccess', $custom_upload_dir . '.htaccess');
        @copy(dirname(__FILE__) . '/files/web.config', $custom_upload_dir . 'web.config');
        
        update_option('file-commerce-pro-version', FILE_COMMERCE_PRO_VERSION);
    }

    function hook_deactivate() {
        
    }

    // AJAX CALLBACK //

    /**
     * Admin ajax callback
     */
    function admin_ajax_action_callback() {
        global $wpdb; // this is how you get access to the database
        //get event value -> every ajax call must have a different one
        $event = $_GET['event'];

        if ($event == 'test') {
            $result = '{"status":"ok","msg":"this is only a test"}';
        } elseif ($event == 'loadcontractsdata') {
            $months = array();
            $counts = array();

            //last 12 months sells and last 12 months text array
            $query = "select concat(year(created), '-', date_format(created, '%m')) as d, count(id) as c
                                from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS . " where status='" . FileCommercePro::CONTRACT_VALID . "'";
            if (!empty($_GET['fileid'])) {
                $query .= " and fileid=" . $wpdb->escape($_GET['fileid']);
                $file = $wpdb->get_row($wpdb->prepare("select * from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_FILES . " where id=%d", $_GET['fileid']), ARRAY_A);
                $title = $file['title'];
            } else {
                $title = 'Overall sells';
            }
            $query .= " group by concat(year(created), '-', date_format(created, '%m')) order by d desc limit 12";

            $array = $wpdb->get_results($query, 'ARRAY_A');

            $array = array_reverse($array);
            foreach ($array as $key => $value) {
                $months[] = '"' . $value['d'] . '"';
                $counts[] = $value['c'];
            }
            $months = implode(",", $months);
            $counts = implode(",", $counts);

            $result = '{"status": "ok", "title": "' . addslashes($title) . '", "months": [' . $months . '], "counts": [' . $counts . ']}';
        } elseif ($event == 'loaddownloadsdata') {
            $titles = array();
            $hits = array();

            //last 12 months downloads for file
            $array = file_commerce_pro_repo_get_download_hits();
            foreach ($array as $key => $value) {
                $titles[] = '"' . $value['title'] . '"';
                $hits[] = $value['hits'];
            }
            $titles = implode(",", $titles);
            $hits = implode(",", $hits);

            $result = '{"status": "ok", "titles": [' . $titles . '], "hits": [' . $hits . ']}';
        } else {
            $result = '{"status":"ko","msg":"no event specified"}';
        }

        echo $result;

        die(); // this is required to return a proper result
    }

    /*     * **********************************
      PAYMENTS
     * ********************************** */

    /**
     * Redirect user to clickbank payment page
     * @param $fileid
     */
    function clickbank_redirect($fileid, $userid) {
        if (is_numeric($fileid) && is_numeric($userid)) {
            //load file
            $file = file_commerce_pro_load_file_from_db($fileid);
            if ($file->status > 0)
                    header("Location: http://" . $file->clickbank . "." . $this->get_option($this->option_name_clickbank_vendor_code) . ".pay.clickbank.net/?user=" . $userid);
            exit();
        }
        die('Access denied');
    }

    /**
     * Returns contracts expiration date in UNIX timestamp
     * @param $file_duration duration value for the file
     */
    function get_contract_expiration_value($file_duration) {
        return time() + $file_duration * 24 * 3600;
    }

    /**
     * Generate an "From" header, if needed, to get the name and email from main configuration. If an email
     * is not specified an empty tring is returned so the mail will be sent with the default WordPress
     * "From" header.
     */
    function mail_from_header() {
        $sender_email = $this->get_option('sender_email');
        if (!empty($sender_email)) {
            return 'From: ' . $this->get_option('sender_name') . ' <' . $sender_email . '>' . "\n";
        }
        return '';
    }

    /**
     * Send out an email, logging it.
     */
    function mail($to, $subject, $body, $headers='') {
        $this->log_debug('Mailing to ' . $to);
        $this->log_debug('Subject: ' . $subject);
        $this->log_debug('Headers: ' . $headers);

        if (strpos($headers, 'From:') === false) {
            $sender_email = $this->get_option('sender_email');
            if (!empty($sender_email)) {
                $headers .= 'From: ' . $this->get_option('sender_name') . ' <' . $sender_email . '>' . "\n";
            }
        }

        $r = wp_mail($to, $subject, $body, $headers);
        if (!$r) $this->log_info('Mail failed');
        return $r;
    }

    /* Send an email to the site administrator, as From, if not set on header, is used the default WordPress From */

    function admin_mail($subject, $body, $headers='') {
        $this->log_debug('Mailing to ' . $to);
        $this->log_debug('Subject: ' . $subject);
        $this->log_debug('Headers: ' . $headers);
        $r = wp_mail(get_option('admin_email'), $subject, $body, $headers);
        if (!$r) $this->log_info('Admin mail failed');
        return $r;
    }

    /**
     * Notify the customer and the admin of a new contract creation by email
     */
    function notify_new_contract($contract, $user = null) {

        // Controlla se a livello di file è specificato un messaggio di notifica per il nuovo acquisto
        // altrimenti utilizza quello della configurazione principale.
        $file = $this->get_file(is_object($contract) ? $contract->fileid : $contract['fileid']);
        if (empty($file->notify_ok_subject)) {
            $subject = $this->get_option('notify_ok_subject');
            $body = $this->get_option('notify_ok_body');
        } else {
            $subject = $file->notify_ok_subject;
            $body = $file->notify_ok_body;
        }

        $body = $this->replace($body, $contract, $user);
        $user = get_user_by('id', is_object($contract) ? $contract->userid : $contract['userid']);
        // TODO: add the BCC if set on main configuration
        $this->mail($user->user_email, $subject, $body);

        // Messaggio per l'amministratore
        $message = "A new file has just been sold:\n\n";
        $message .= "File\n";
        $message .= " - file id: {file_id}\n";
        $message .= " - file title: {file_title}\n";
        $message .= "\n";
        $message .= "Contract\n";
        $message .= " - contract id: {contract_id}\n";
        $message .= " - contract expire: {contract_expire}\n";
        $message .= " - payment gateway: {contract_gateway}\n";
        $message .= "\n";
        $message .= "User details\n";
        $message .= " - user login: {user_login}\n";
        $message .= " - user email: {user_email}\n";
        $message .= " - user id: {user_id}\n";
        $message .= "\n";
        $message .= "Excellent, yours File Commerce Pro";

        $message = $this->replace($message, $contract);
        $this->admin_mail('[' . get_option('blogname') . '] New file sold', $message);
    }

    /**
     * Notify a contract invalidation by email to customer and to admin
     */
    function notify_invalidated_contract($contract) {
        // Notify the user about the new contract status
        $body = $this->get_option('notify_ko_body');
        $body = $this->replace($body, $contract);
        $user = get_user_by('id', is_object($contract) ? $contract->userid : $contract['userid']);
        $this->mail($user->user_email, $this->get_option('notify_ko_subject'), $body);

        // Notify the admin
        $message = "A contract has been invalidated:\n\n";
        $message .= "file: {file_title}\n";
        $message .= "user login: {user_login}\n";
        $message .= "user email: {user_email}\n";
        $message .= "user id: {user_id}\n";
        $message .= "gateway: {contract_gateway}\n";
        $message .= 'reason: ' . $reason . "\n";

        $message .= "\n";
        $message .= "Don't worry for one lost two gained. Yours, File Commerce Pro";

        $message = $this->replace($message, $contract);

        $this->admin_mail('[' . get_option('blogname') . '] Contract invalidated', $message);
    }

    /** This is a generic replacement function that can be used anywhere. To replace only user data, pass only the
     * $user. If a contract (both object or array) is passed and the user and file are null, they will be extracted
     * automatically.
     */
    function replace($text, $contract = null, $user = null, $file = null) {
        if ($contract != null) {
            if ($user == null)
                    $user = get_user_by('id', is_object($contract) ? $contract->userid : $contract['userid']);
            if ($file == null)
                    $file = file_commerce_pro_load_file_from_db(is_object($contract) ? $contract->fileid : $contract['fileid']);
        }

        if ($file != null) {
            $text = str_replace('{file_title}', strip_tags($file->title), $text);
            $text = str_replace('{file_id}', strip_tags($file->id), $text);
        }

        if ($contract != null) {
            $contract_id = is_object($contract) ? $contract->id : $contract['id'];
            $contract_token = is_object($contract) ? $contract->token : $contract['token'];
            $contract_key = $contract_id . '-' . $contract_token;
            $text = str_replace('{contract_expire}', date('Y/m/d', is_object($contract) ? $contract->expire : $contract['expire']), $text);
            $text = str_replace('{contract_gateway}', is_object($contract) ? $contract->gateway : $contract['gateway'], $text);
            $text = str_replace('{contract_id}', $contract_id, $text);
            $text = str_replace('{contract_key}', $contract_key, $text);

            if (!empty($contract_token)) {
                $text = str_replace('{file_url}', plugins_url('get.php', __FILE__) . '?k=' . $contract_key, $text);
            } else {
                $text = str_replace('{file_url}', $this->get_option('main_url'), $text);
            }

            $custom = is_object($contract) ? $contract->custom : $contract['custom'];
            $php_code = $this->get_option('extend_custom_display_email');
            if (!empty($php_code)) $custom = eval($php_code);

            $text = str_replace('{contract_custom}', $custom, $text);
        }

        if ($user != null) {
            $text = str_replace('{user_id}', $user->ID, $text);
            $text = str_replace('{user_login}', $user->user_login, $text);
            $text = str_replace('{user_email}', $user->user_email, $text);
            // Fake field force only on new user creation
            if (isset($user->password)) {
                $text = str_replace('{user_password}', $user->password, $text);
            } else {
                $text = str_replace('{user_password}', '*****', $text);
            }
        }
        // Generic login, right for emails too
        $text = str_replace('{login_url}', wp_login_url(get_option('home')), $text);
        $text = str_replace('{password_url}', site_url('wp-login.php?action=lostpassword', 'login'), $text);

        if (!is_user_logged_in()) {
            $text = str_replace('{login_url_download}', wp_login_url($this->get_option('main_url')), $text);
        } else {
            $text = str_replace('{login_url_download}', $this->get_option('main_url'), $text);
        }

        $text = str_replace('{download_url}', $this->get_option('main_url'), $text);

        // Login that redirects to the current page
        $text = str_replace('{login_url_back}', wp_login_url(get_permalink()), $text);

        return $text;
    }

    /**
     * Returns a list of ACTIVE contract for the user and the file passed. Active contracts
     * are the ones with VALID status and NOT EXPIRED.
     */
    function get_active_contracts($user_id, $file_id) {
        global $wpdb;

        return $wpdb->get_results("select * from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_CONTRACTS .
                        " where fileid = " . $file_id . " and userid = " . $user_id . " and status = " .
                        FileCommercePro::CONTRACT_VALID . " and expire > " . time() . " order by expire desc");
    }

    function hook_wp_head() {
        echo '<link rel="stylesheet" type="text/css" href="' . plugins_url('/css/style.css', __FILE__) . '"/>';
        echo $this->get_option('style');
    }

    /**
     * Returns wp upload dir base path, without trailing slash
     */
    function get_wp_upload_dir_path() {
        $uploads = wp_upload_dir();
        return $uploads['basedir'];
    }

    /**
     * Returns wp upload dir base url, without trailing slash
     */
    function get_wp_upload_dir_url() {
        $uploads = wp_upload_dir();
        return $uploads['baseurl'];
    }

    function create_user($email, $first_name='', $last_name='') {
        global $wpdb;

        $this->log_debug('Creating/checking user with email: ' . $email);

        // Checks if there is already an user registered with this email/login
        $user = get_user_by('login', $email);
        if (!$user) $user = get_user_by('email', $email);

        if ($user) {
            $this->log_debug('User already registered: ' . $user->ID . ' - ' . $email);
            return $user;
        }

        $user = array();

        $user['user_login'] = $email;
        $user['user_nicename'] = sanitize_title($email);
        $user['user_url'] = '';
        $user['user_email'] = $email;
        $user['display_name'] = 'Anonymous';
        $password = wp_generate_password();
        $user['user_pass'] = wp_hash_password($password);
        $user['user_registered'] = gmdate('Y-m-d H:i:S');

        $wpdb->insert($wpdb->users, $user);
        $user_id = $wpdb->insert_id;

        $this->log_debug('User created: ' . $user_id);

        update_user_meta($user_id, 'first_name', $first_name);
        update_user_meta($user_id, 'last_name', $last_name);
        update_user_meta($user_id, 'nickname', $email);
        update_user_meta($user_id, 'description', '');
        update_user_meta($user_id, 'rich_editing', 'true');
        update_user_meta($user_id, 'comment_shortcuts', 'false');
        update_user_meta($user_id, 'admin_color', 'fresh');
        update_user_meta($user_id, 'use_ssl', 0);

        $user = new WP_User($user_id);
        $user->set_role(get_option('default_role'));
        //$user->set_role(FileCommercePro::ROLE_NAME);

        $user = get_user_by('id', $user_id);

        // Fake field for email composing
        $user->password = $password;

        // Send an email
        $body = $this->get_option('new_user_body');
        $body = $this->replace($body, null, $user);

        $this->mail($user->user_email, $this->get_option('new_user_subject'), $body);

        return $user;
    }

    function paypal_button($file) {
        global $current_user;

        if ($file->paypal_enabled != 1) return '';

        $buffer = '';
        if (!empty($file->paypal_form)) {
            $buffer = $file->paypal_form;
            $hidden = '<input type="hidden" name="notify_url" value="' . plugins_url('paypal.php', __FILE__) . '"/>';
            $hidden .= '<input type="hidden" name="return" value="' . plugins_url('thankyou.php', __FILE__) . '"/>';
            $buffer = str_ireplace('</form>', $hidden . '</form>', $buffer);
        } else if (!empty($file->paypal) || !empty($file->amount)) {
            $buffer = '<a href="' . $this->paypal_button_url($file, $current_user->ID) . '">';
            $button = $this->get_option('plimus_button');
            if (empty($button)) {
                $buffer .= '<img src="' . plugins_url('img/paypal.jpg', __FILE__) . '"/>';
            }
            else {
                $buffer .= $button;
            }
            $buffer .= '</a> ';
        }
        return $buffer;
    }

    function clickbank_button_url($file) {
        return plugins_url('buy.php', __FILE__) . '?g=clickbank&amp;f=' . $file->id;
    }

    function clickbank_button($file) {
        if ($file->clickbank_enabled == 0) return '';
        
        $buffer .= '<a href="' . $this->clickbank_button_url($file) . '">';
        $button = $this->get_option('clickbank_button');
        if (empty($button)) {
            $buffer .= '<img src="' . plugins_url('img/clickbank.jpg', __FILE__) . '"/>';
        }
        else {
            $buffer .= $button;
        }
        $buffer .= '</a> ';
        return $buffer;
    }

    function plimus_button_url($file) {
        return plugins_url('buy.php', __FILE__) . '?g=plimus&amp;f=' . $file->id;
    }

    function plimus_button($file) {
        $buffer .= '<a href="' . $this->plimus_button_url($file) . '">';
        $button = $this->get_option('plimus_button');
        if (empty($button)) {
            $buffer .= '<img src="' . plugins_url('img/plimus.jpg', __FILE__) . '"/>';
        }
        else {
            $buffer .= $button;
        }
        $buffer .= '</a> ';
        return $buffer;
    }

    function paypal_button_url($file, $user_id) {
        return plugins_url('buy.php', __FILE__) . '?g=paypal&amp;f=' . $file->id;
    }

    /**
     * Lead a file from database with given id, not status check is made. Format can be
     * changed from OBJECT to ARRAY_A to have an associative array instead of an object.
     */
    function get_file($id, $format=OBJECT) {
        global $wpdb;
        $query = $wpdb->prepare("select * from " . $wpdb->prefix . FILECOMMERCEPRO_TABLE_FILES . " where id=%d", $id);
        return $wpdb->get_row($query, $format);
    }

    function get_token($prefix='') {
        $token = $prefix . md5(rand(0, time()));
    }

}

@include_once dirname(__FILE__) . '/widget.php';